/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Checkbox, Icon} from "semantic-ui-react";
import {TestTypeFullBase} from "../../../../types/testTypes";
import {RunnableCustomTestFrontendExclusive} from "../../../../types/customTests";
import {RunnableExerciseTestFrontendExclusive} from "../../../../types/exerciseEditor";
import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {AssessedSubmitTestFromBackend, TestResultCodes} from "../../../../types/testResults";
import {CheckboxOnChangeEventData} from "../../../../types/reactEvents";
import {HelpPopup} from "../../../helpers/helpPopup";
import {OkCircularIcon} from "../../../helpers/okCircularIcon";
import {ErrorCircularIcon} from "../../../helpers/errorCircularIcon";
import {getI18n} from "../../../../../i18n/i18nRoot";
import {TestProtocol} from '../../../../types/testsAndTestProtocol'
import {startsWith} from '../../../../helpers/stringHelper'
import {MyPopup} from '../../../helpers/myPopup'
import {
  setUseDiffIfOutputMismatched,
  setUseReplaceWhiteSpacesWithBullets
} from '../../../../state/actions/globalUiActions'
import {testServerPublicSettingsNotAvailableValue} from '../../../../constants'


//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly test: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFromBackend | null
  readonly testTypes: ReadonlyArray<TestTypeFullBase>
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
    useDiffIfOutputMismatched: rootState.globalUiState.useDiffIfOutputMismatched,
    useReplaceWhiteSpacesWithBullets: rootState.globalUiState.useReplaceWhiteSpacesWithBullets,
    hideExpectedOutputLines: rootState.globalUiState.hideExpectedOutputLines,
    testServerPublicSettings: rootState.globalSystemSettingsState.testServerPublicSettings,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setUseDiffIfOutputMismatched,
                                                                        setUseReplaceWhiteSpacesWithBullets,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class consolePanelHeaderBar extends React.Component<Props, any> {


  render(): JSX.Element {


    // noinspection TsLint
    let hasMaxErrorLinesHit: boolean = false
    // noinspection TsLint
    let hasMaxTestInputLinesHit: boolean = false
    // noinspection TsLint
    let hasMaxProgramOutputLinesHit: boolean = false

    const minMaxLinesValue = Math.min(this.props.testServerPublicSettings.maxErrLinesToRead,
                                      this.props.testServerPublicSettings.maxLinesToWrite,
                                      this.props.testServerPublicSettings.maxLinesToRead
    )

    // noinspection TsLint
    if (this.props.test && this.props.test.result && this.props.test.result.protocol.length >= minMaxLinesValue && minMaxLinesValue !== testServerPublicSettingsNotAvailableValue) {


      const numMaxErrorLinesHit: number = this.props.test.result.protocol.reduce(
        (p, c) => p + (startsWith(c.trim(), TestProtocol.errorPrefix)
                       ? 1
                       : 0),
        0
      )
      const numMaxTestInputLinesHit: number = this.props.test.result.protocol.reduce(
        (p, c) => p + (startsWith(c.trim(), TestProtocol.inputSign)
                       ? 1
                       : 0),
        0
      )
      const numMaxProgramOutputLinesHit: number = this.props.test.result.protocol.reduce(
        (p, c) => p + (startsWith(c.trim(), TestProtocol.outputSign)
                       ? 1
                       : 0),
        0
      )

      hasMaxErrorLinesHit = numMaxErrorLinesHit >= this.props.testServerPublicSettings.maxErrLinesToRead
      hasMaxTestInputLinesHit = numMaxTestInputLinesHit >= this.props.testServerPublicSettings.maxLinesToWrite
      hasMaxProgramOutputLinesHit = numMaxProgramOutputLinesHit >= this.props.testServerPublicSettings.maxLinesToRead
    }

    return (
      <div className="flexed">
        {
          (this.props.test && this.props.test.result) &&
          <div className="flexed-no-wrap">

            <div className="flexed-no-wrap not-text-wrapping">
              <div className="mar-right-half v-centered">
                {getI18n(this.props.langId, 'State')}:
              </div>

              <div>
                {
                  //this is somehow a hack because this is from the original (without react) semantic ui
                  this.props.test.result.passed &&
                  <OkCircularIcon/>
                }
                {
                  !this.props.test.result.passed &&
                  <ErrorCircularIcon/>
                }
              </div>

              <div className="v-centered">
                {
                  this.props.test.result.testResultCode === TestResultCodes.CompileError &&
                  <div>
                    {
                      getI18n(this.props.langId, 'Compiler error')
                    }
                  </div>
                }
                {
                  this.props.test.result.testResultCode === TestResultCodes.OutputMismatch &&
                  <div>
                    {
                      getI18n(this.props.langId, 'Wrong output')
                    }
                  </div>
                }
                {
                  this.props.test.result.testResultCode === TestResultCodes.UnknownTestMethod &&
                  <div>
                    {
                      getI18n(this.props.langId, 'Unknown test type')
                    }
                  </div>
                }
                {
                  this.props.test.result.testResultCode === TestResultCodes.TimeoutHit &&
                  <div>
                    <span>{
                      getI18n(this.props.langId, 'Program timeout')
                    }
                    </span>
                    <HelpPopup className="mar-left-half" wide
                               defaultText={'A timeout can have 3 main reasons. 1. Your program has an infinite loop (not terminating) 2. Your program tries to read from the console but no input is provided from the test (waiting forever for input) 3. The test(-runner) expects your program to output something (waiting forever for output) but your program doesn\'t write anything (so you output less than the test expects)'}/>
                  </div>
                }
                {
                  this.props.test.result.testResultCode === TestResultCodes.DiskLimitHit &&
                  <div>
                    {
                      getI18n(this.props.langId, 'Disk space limit')
                    }
                  </div>
                }
                {
                  this.props.test.result.testResultCode === TestResultCodes.MemoryLimitHit &&
                  <div>
                    {
                      getI18n(this.props.langId, 'RAM limit')
                    }
                  </div>
                }
                {
                  this.props.test.result.testResultCode === TestResultCodes.CompileTimeoutHit &&
                  <div>
                    <span>{
                      getI18n(this.props.langId, 'Compile timeout')
                    }
                    </span>
                  </div>
                }
                {
                  this.props.test.result.testResultCode === TestResultCodes.SomeServerError &&
                  <div>
                    {
                      getI18n(this.props.langId, 'Unknown test-server error')
                    }
                  </div>
                }
                {
                  this.props.test.result.testResultCode === TestResultCodes.UserProgramError &&
                  <div>
                    {
                      getI18n(this.props.langId, 'User program error')
                    }
                  </div>
                }
                {
                  this.props.test.result.testResultCode === TestResultCodes.TestContentReadTimeout &&
                  <div>
                    {
                      getI18n(this.props.langId, 'Test content read timeout')
                    }
                  </div>
                }
                {
                  this.props.test.result.testResultCode === TestResultCodes.ExitCodeMismatch &&
                  <div>
                    {
                      getI18n(this.props.langId, 'Exit code mismatch')
                    }
                  </div>
                }
                {
                  this.props.test.result.testResultCode === TestResultCodes.CompareFilesMismatch &&
                  <div>
                    {
                      getI18n(this.props.langId, 'Compare files mismatch')
                    }
                  </div>
                }

              </div>

              <SimpleVDivider/>

              <div className="v-centered not-text-wrapping">
                <div className="flexed-no-wrap not-text-wrapping">
                  <Checkbox className="mar-right-half"
                            label={getI18n(this.props.langId, 'Use diff')}
                            checked={this.props.useDiffIfOutputMismatched}
                            onChange={(e, data: CheckboxOnChangeEventData) => {
                              this.props.setUseDiffIfOutputMismatched(data.checked)
                            }}
                  />
                  <HelpPopup defaultText={getI18n(this.props.langId,
                                                  'If enabled and your output was wrong then the diff against the expected output is displayed as your output. If disabled the wrong output is marked red instead. For further help (if enabled) hover over the rectangle left to the expected output.'
                  )}
                             wide
                  />
                </div>
              </div>

              <SimpleVDivider/>

              <div className="v-centered not-text-wrapping">
                <div className="flexed-no-wrap not-text-wrapping">
                  <Checkbox className="mar-right-half"
                            label={getI18n(this.props.langId, 'Replace Spaces')}
                            checked={this.props.useReplaceWhiteSpacesWithBullets}
                            onChange={(e, data: CheckboxOnChangeEventData) => {
                              this.props.setUseReplaceWhiteSpacesWithBullets(data.checked)
                            }}
                  />
                  <HelpPopup defaultText={getI18n(this.props.langId,
                                                  'Replaces all whitespaces with · and all tabs with ⇥ in the input and output'
                  )}/>
                </div>
              </div>

            </div>

            <SimpleVDivider/>

            <div className="v-centered flexed-no-wrap too-much-text-hidden">

              {
                (hasMaxErrorLinesHit || hasMaxProgramOutputLinesHit || hasMaxTestInputLinesHit) &&
                <MyPopup
                  trigger={
                    <div className="mar-right-half hoverable">
                      <Icon className="error-colored" name="exclamation triangle"/>
                    </div>
                  }
                  content={
                    <div>
                      <div>{getI18n(this.props.langId, "The test protocol contained too many lines...")}</div>
                      {
                        hasMaxTestInputLinesHit &&
                        <div>{
                          `${getI18n(this.props.langId,
                                     "The stdin (test input lines) contains more than (or equal to)"
                          )} ${this.props.testServerPublicSettings.maxErrLinesToRead} ${getI18n(this.props.langId,
                                                                                                "lines. All lines after the"
                          )} ${this.props.testServerPublicSettings.maxErrLinesToRead} ${getI18n(this.props.langId,
                                                                                                "-th are just dropped (technical reasons)."
                          )}`
                        }</div>
                      }
                      {
                        hasMaxProgramOutputLinesHit &&
                        <div>{
                          `${getI18n(this.props.langId,
                                     "The stdout (program output) contains more than (or equal to)"
                          )} ${this.props.testServerPublicSettings.maxErrLinesToRead} ${getI18n(this.props.langId,
                                                                                                "lines. All lines after the"
                          )} ${this.props.testServerPublicSettings.maxErrLinesToRead} ${getI18n(this.props.langId,
                                                                                                "-th are just dropped (technical reasons)."
                          )}`
                        }</div>
                      }
                      {
                        hasMaxErrorLinesHit &&
                        <div>{
                          `${getI18n(this.props.langId,
                                     "The stderr (program error output) contains more than (or equal to)"
                          )} ${this.props.testServerPublicSettings.maxErrLinesToRead} ${getI18n(this.props.langId,
                                                                                                "lines. All lines after the"
                          )} ${this.props.testServerPublicSettings.maxErrLinesToRead} ${getI18n(this.props.langId,
                                                                                                "-th are just dropped (technical reasons)."
                          )}`
                        }</div>
                      }
                    </div>
                  }
                  wide="very"
                />
              }

              <div className="mar-right-half">
                {getI18n(this.props.langId, 'Test')}:
              </div>
              <div>
                {this.props.test.displayName}
              </div>
              {
                //the case is because AssessedSubmitTestFromBackend don't has this prop
                this.props.test && (this.props.test as RunnableExerciseTestFrontendExclusive).uiIsAfterTest &&
                <div className="mar-left-half"><Icon name="life ring"></Icon></div>
              }
            </div>
          </div>
        }
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(consolePanelHeaderBar)
