import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from '../../../../state/reducers'
import SimpleCodeEditorWrapper from '../../../codeEditors/simpleCodeEditorWrapper'
import {tutorFeedbackEditorDebounceInMs} from '../../../../constants'
import {setFeedbackForStudent} from '../../../../state/actions/tutorViewSite/sub/setAssessmentActions'
import {EditorTab} from '../../../codeEditors/multiTabCodeEditorProps'
import {KnownInternalTestTypes} from '../../../../types/testTypes'
import {
  aceCompareFilesTestLanguage,
  aceRegexLanguage,
  aceTestProtocolLanguage,
  aceTextLanguage
} from '../../../../helpers/editorHelper'

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    releaseFinishedAndAutomaticAssessmentHasFinished: !rootState.tutorViewSiteState.exercise.exerciseRelease.isReleased &&
      rootState.tutorViewSiteState.exercise.exerciseRelease.hasAutomaticAssessmentFinished,
    assessment: rootState.tutorViewSiteState.assessment,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setFeedbackForStudent,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


const tutorFeedbackEditor = 'tutorFeedbackEditor'

class FeedbackEditorView extends React.Component<Props, any> {
  render(): JSX.Element {

    const tempEditorTab: EditorTab = {
      id: 0,
      content: (this.props.assessment !== null && this.props.assessment.feedbackForStudent !== null)
               ? this.props.assessment.feedbackForStudent
               : '',
      language: 'markdown',
      isReadonly: this.props.releaseFinishedAndAutomaticAssessmentHasFinished === false,
      fileNameWithExtensions: '',
      displayIndex: 1,
      isMainFile: true,
      isContentVisibleForUser: true,
    }

    return (
      <div className="fh" style={{overflow: 'hidden'}}>

        <SimpleCodeEditorWrapper
          id={tutorFeedbackEditor}
          editorTab={tempEditorTab}
          onBlur={(content: string, editorTabId: number) => {
            this.props.setFeedbackForStudent(content)
          }}
          debounceTimeInMs={tutorFeedbackEditorDebounceInMs}

        />

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(FeedbackEditorView)
